const { Client, LocalAuth } = require('whatsapp-web.js');
const qrcode = require('qrcode-terminal');
const dotenv = require('dotenv');
const weatherUtils = require('./utils/weather_utils');
const weatherService = require('./services/weather_service');
const logger = require('./utils/logger');
const { handleRecipeCommand } = require('./commands/recipes_command');
const { handleWeatherCommand } = require('./commands/weather_command');
const { initializeCronJobs } = require('./services/cron_service');
const { handleGuideCommand, handleLocationMessage } = require('./commands/guide_command');
const { handleStoicismCommand } = require('./commands/stoicism_command');

dotenv.config();

// Constantes
const CHROME_PATH = process.env.CHROME_PATH;
const AUTHORIZED_NUMBER = process.env.AUTHORIZED_NUMBER;

// Validación inicial
if (!AUTHORIZED_NUMBER) {
    console.error('❌ Debes establecer el número autorizado en el archivo .env');
    process.exit(1);
}

const clientConfig = {
    authStrategy: new LocalAuth(),
    puppeteer: {
        headless: true,
        executablePath: CHROME_PATH,
        args: ['--no-sandbox', '--disable-setuid-sandbox', '--disable-dev-shm-usage']
    }
};

class WhatsAppBot {
    constructor() {
        this.client = new Client(clientConfig);
        this.myId = null;
        this.myNumber = null;
        this.initializeEventHandlers();
    }

    initializeEventHandlers() {
        this.client.on('qr', this.handleQR);
        this.client.on('ready', this.handleReady.bind(this));
        this.client.on('message_create', this.handleMessage.bind(this));
        this.client.on('message', this.logMessage);
        this.client.on('group_join', this.handleGroupJoin);
    }

    handleQR(qr) {
        qrcode.generate(qr, { small: true });
        logger.info('🔍 Escanea el QR code con tu WhatsApp.');
    }

    async handleReady() {
        logger.info('✅ Cliente de WhatsApp está listo!');
        try {
            this.myId = this.client.info.wid._serialized;
            this.myNumber = this.myId.split('@')[0];
            logger.info(`🔑 Tu ID de WhatsApp es: ${this.myId}`);
            logger.info(`📱 Tu número de teléfono es: ${this.myNumber}`);

            initializeCronJobs(this.client);

            const chat = await this.client.getChatById(this.myId);
            //logger.info(`💭 Chat propio encontrado: ${JSON.stringify(chat, null, 2)}`);
        } catch (error) {
            logger.error(`❌ Error al obtener información: ${error}`);
        }
    }

    isChatAllowed(messageFrom) {
        const isAllowed = messageFrom === AUTHORIZED_NUMBER;
        if (isAllowed) console.log('🎯 Mensaje detectado desde el número autorizado');
        return isAllowed;
    }

    async handleMessage(message) {
        if (!this.myId || !this.isChatAllowed(message.from)) return;
    
        if (message.type === 'location') {
            handleLocationMessage(this.client, message);
            return;
        }
    
        console.log('📨 Mensaje creado:', {
            from: message.from,
            fromMe: message.fromMe,
            body: message.body,
            type: message.type,
            timestamp: message.timestamp
        });
    
        const command = message.body.trim().toLowerCase();
        
        const commands = {
            '!ping': () => this.handlePingCommand(message),
            '!clima': () => handleWeatherCommand(this.client, message),
            '!receta': () => handleRecipeCommand(this.client, message),
            '!guiaciudad': () => handleGuideCommand(this.client, message),
            '!calm': () => handleStoicismCommand(this.client, message)
        };
    
        const commandHandler = Object.entries(commands).find(([key]) => command.startsWith(key));
        
        if (commandHandler) {
            await commandHandler[1]();
        }
    }   

    async handlePingCommand(message) {
        try {
            await this.client.sendMessage(message.from, 'pong madafaka');
            console.log(`📤 Respondido 'pong' al número autorizado ${message.from}`);
        } catch (error) {
            console.error('❌ Error al enviar respuesta:', error);
        }
    }

    logMessage(message) {
        console.log('📩 Mensaje recibido:', {
            from: message.from,
            fromMe: message.fromMe,
            body: message.body
        });
    }

    handleGroupJoin(notification) {
        console.log('👥 Bot añadido a un nuevo grupo:', notification.id.remote);
    }

    start() {
        this.client.initialize();
    }
}

const bot = new WhatsAppBot();
bot.start();
